Method: ShellOpts::Command#__option_values__
- Defined in:
- lib/shellopts/program.rb
#__option_values__ ⇒ Object (readonly)
Hash from identifier to value. Can be Integer, Float, or String depending on the option’s type. Repeated options options without arguments have the number of occurences as value, repeated option with arguments have the array of values as value
163 164 165 |
# File 'lib/shellopts/program.rb', line 163 def __option_values__ @__option_values__ end |